////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.facebook;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.Calendar;
import org.watto.android.Language;
import org.watto.android.R;
import org.watto.program.android.sync.facebook.array.ArrayResizer;
import org.watto.program.android.sync.facebook.datatype.BiographyData;
import org.watto.program.android.sync.facebook.datatype.CheckInData;
import org.watto.program.android.sync.facebook.datatype.GenderData;
import org.watto.program.android.sync.facebook.datatype.PoliticsData;
import org.watto.program.android.sync.facebook.datatype.RelationshipData;
import org.watto.program.android.sync.facebook.datatype.ReligionData;
import org.watto.program.android.sync.facebook.datatype.StatusData;
import org.watto.program.android.sync.facebook.datatype.UserIdData;
import android.content.ContentProviderOperation;
import android.content.ContentValues;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.provider.ContactsContract;
import android.provider.ContactsContract.CommonDataKinds.Email;
import android.provider.ContactsContract.CommonDataKinds.Event;
import android.provider.ContactsContract.CommonDataKinds.GroupMembership;
import android.provider.ContactsContract.CommonDataKinds.Nickname;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.CommonDataKinds.Photo;
import android.provider.ContactsContract.CommonDataKinds.StructuredName;
import android.provider.ContactsContract.CommonDataKinds.StructuredPostal;
import android.provider.ContactsContract.CommonDataKinds.Website;
import android.provider.ContactsContract.Data;
import android.provider.ContactsContract.Groups;
import android.provider.ContactsContract.RawContacts;


/***********************************************************************************************
Helper class for storing data in the platform content providers
***********************************************************************************************/
public class ContactOperations {

  ContentValues content;
  ContentProviderOperation.Builder operationBuilder;
  BatchOperation batchOperation;
  Context context;
  boolean yield;
  long rawContactId = 0;
  int backReference;
  boolean newContact;


  /***********************************************************************************************
  Constructor
  ***********************************************************************************************/
  public ContactOperations(Context context, BatchOperation batchOperation){
    this.content = new ContentValues();
    this.yield = true;
    this.context = context;
    this.batchOperation = batchOperation;
  }


  /***********************************************************************************************
  Constructor for adding a new User
  ***********************************************************************************************/
  public ContactOperations(Context context, int userId, String accountName, BatchOperation batchOperation){
    this(context,batchOperation);
    this.backReference = batchOperation.size();
    this.newContact = true;

    content.put(RawContacts.SOURCE_ID,userId);
    content.put(RawContacts.ACCOUNT_TYPE,GlobalConstants.ACCOUNT_TYPE);
    content.put(RawContacts.ACCOUNT_NAME,accountName);

    this.operationBuilder = addInsertOperation(RawContacts.CONTENT_URI,true).withValues(content);
    batchOperation.add(operationBuilder.build());
  }


  /***********************************************************************************************
  Constructor for adding a new Group
  ***********************************************************************************************/
  public ContactOperations(Context context, int groupId, String accountName, BatchOperation batchOperation, boolean addingGroup){
    this(context,batchOperation);
    this.backReference = batchOperation.size();
    this.newContact = true;
  }


  /***********************************************************************************************
  Constructor for updating an existing User
  ***********************************************************************************************/
  public ContactOperations(Context context, long rawContactId, BatchOperation batchOperation){
    this(context,batchOperation);
    this.newContact = false;
    this.rawContactId = rawContactId;
  }


  /***********************************************************************************************
  Adds the users biography
  @param biography the Biography
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addBiography(String biography){
    content.clear();
    if (biography != null && !biography.equals("")) {
      content.put(BiographyData.DATA,biography);
      content.put(BiographyData.LABEL,context.getString(R.string.label_biography));
      content.put(Data.MIMETYPE,BiographyData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds the birthday
  @param birthday the birthday as MM/DD/YYYY or MM/DD/YYYY
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addBirthday(String birthday){
    content.clear();
    if (birthday != null && !birthday.equals("")) {
      content.put(Event.START_DATE,birthday);
      content.put(Event.TYPE,Event.TYPE_BIRTHDAY);
      content.put(Event.LABEL,context.getString(R.string.label_birthday));
      content.put(Data.MIMETYPE,Event.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a checkin
  @param checkin the checkin
  @param checkinTime the checkin time
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addCheckin(String checkin,long checkinTime){
    content.clear();
    if (checkin != null && !checkin.equals("")) {
      content.put(Data.MIMETYPE,CheckInData.CONTENT_ITEM_TYPE);
      content.put(CheckInData.LABEL,context.getString(R.string.label_checkin));
      content.put(CheckInData.VENUE,checkin);
      content.put(CheckInData.TIMESTAMP,checkinTime);

      // Build the checkin message
      String checkinMessage = "";

      if (checkin != null && !checkin.equals("")) {
        checkinMessage += checkin;
      }

      if (checkinTime > 0) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(checkinTime);

        if (!checkinMessage.equals("")) {
          checkinMessage += " at " + formatTimestamp(calendar);
        }
      }

      if (!checkinMessage.equals("")) {
        content.put(CheckInData.DATA,checkinMessage);
      }

    }

    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds an update operation into the batch
  ***********************************************************************************************/
  public void addDeleteOperation(Uri uri){
    operationBuilder = addDeleteOperation(uri,yield);
    yield = false;
    batchOperation.add(operationBuilder.build());
  }


  /***********************************************************************************************
  Adds an email address
  @param new email for user
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addEmail(String email){
    content.clear();
    if (email != null && !email.equals("")) {
      content.put(Email.DATA,email);
      content.put(Email.TYPE,Email.TYPE_OTHER);
      content.put(Email.MIMETYPE,Email.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds the users gender
  @param gender the Gender
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addGender(String gender){
    content.clear();
    if (gender != null && !gender.equals("")) {
      content.put(GenderData.DATA,gender);
      content.put(GenderData.LABEL,context.getString(R.string.label_gender));
      content.put(Data.MIMETYPE,GenderData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a "Facebook" group
  @param title the name of the group
  @param notes a description of the group
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addGroup(String title,String notes){

    content.clear();
    if (title != null && !title.equals("") && notes != null && !notes.equals("")) {
      content.put(Groups.ACCOUNT_TYPE,GlobalConstants.ACCOUNT_TYPE);
      content.put(Groups.ACCOUNT_NAME,GlobalConstants.ACCOUNT_NAME);
      content.put(Groups.TITLE,title);
      content.put(Groups.NOTES,notes);
      content.put(Groups.GROUP_VISIBLE,"1");
      content.put(Groups.SHOULD_SYNC,"1");
    }

    if (content.size() > 0) {
      operationBuilder = addInsertOperation(addCallerIsSyncAdapterParameter(Groups.CONTENT_URI),yield);
      operationBuilder.withValues(content);

      yield = false;
      batchOperation.add(operationBuilder.build());
      // MUST execute here, so the group is created and can be referenced when adding users to the group
      batchOperation.execute();

    }

    return this;

  }


  /***********************************************************************************************
  Adds a contact to a group
  @param groupId the ID of the group to add the contact to
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addGroupMembership(long groupId){
    content.clear();
    if (groupId != -1) {
      content.put(Data.MIMETYPE,GroupMembership.CONTENT_ITEM_TYPE);
      content.put(GroupMembership.GROUP_ROW_ID,groupId);
    }

    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a home city
  @param homeCity the home city
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addHomeCity(String homeCity){
    content.clear();
    if (homeCity != null && !homeCity.equals("")) {
      content.put(StructuredPostal.MIMETYPE,StructuredPostal.CONTENT_ITEM_TYPE);
      content.put(StructuredPostal.FORMATTED_ADDRESS,homeCity);
      content.put(StructuredPostal.TYPE,StructuredPostal.TYPE_CUSTOM);
      content.put(StructuredPostal.LABEL,context.getString(R.string.label_homecity));

      int commaPosition = homeCity.indexOf(',');
      if (commaPosition > 0) {
        String city = homeCity.substring(0,commaPosition);
        content.put(StructuredPostal.CITY,city);
        String state = homeCity.substring(commaPosition + 1).trim();
        content.put(StructuredPostal.REGION,state);
      }
    }

    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds an insert operation into the batch
  ***********************************************************************************************/
  public void addInsertOperation(){

    if (!newContact) {
      content.put(Phone.RAW_CONTACT_ID,rawContactId);
    }

    operationBuilder = addInsertOperation(addCallerIsSyncAdapterParameter(Data.CONTENT_URI),yield);
    operationBuilder.withValues(content);

    if (newContact) {
      operationBuilder.withValueBackReference(Data.RAW_CONTACT_ID,backReference);
    }

    yield = false;
    batchOperation.add(operationBuilder.build());
  }


  /***********************************************************************************************
  Adds where the user is currently living
  @param location the location
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addLocation(String location){
    content.clear();
    if (location != null && !location.equals("")) {
      content.put(StructuredPostal.MIMETYPE,StructuredPostal.CONTENT_ITEM_TYPE);
      content.put(StructuredPostal.FORMATTED_ADDRESS,location);
      content.put(StructuredPostal.TYPE,StructuredPostal.TYPE_CUSTOM);
      content.put(StructuredPostal.LABEL,context.getString(R.string.label_location));

      int commaPosition = location.indexOf(',');
      if (commaPosition > 0) {
        String city = location.substring(0,commaPosition);
        content.put(StructuredPostal.CITY,city);
        String state = location.substring(commaPosition + 1).trim();
        content.put(StructuredPostal.REGION,state);
      }
    }

    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a contact name
  @param name Name of contact
  @param nameType type of name: family name, given name, etc.
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addName(String firstName,String middleName,String lastName){
    content.clear();
    if (firstName != null && !firstName.equals("")) {
      content.put(StructuredName.GIVEN_NAME,firstName);
    }
    if (middleName != null && !middleName.equals("")) {
      content.put(StructuredName.MIDDLE_NAME,middleName);
    }
    if (lastName != null && !lastName.equals("")) {
      content.put(StructuredName.FAMILY_NAME,lastName);
    }
    if (content.size() > 0) {
      content.put(StructuredName.MIMETYPE,StructuredName.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a photo
  @param photoURL the URL to the photo
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addPhoto(URL photoURL){
    content.clear();

    if (photoURL != null && !photoURL.toString().equals("")) {
      // Download the photo into a byte[] BLOB (stored as a String)
      //byte[] photoBytes = downloadPhoto(photoURL);

      // Only add a photo if the download was successful
      //if (photoBytes != null && photoBytes.length > 0) {
        // store the date/time that this photo was retrieved, so its only grabbed once a day
        //String timestamp = "" + Calendar.getInstance().getTimeInMillis();

        // Store the photo
        //content.put(Photo.PHOTO,photoBytes);
        //content.put(Photo.SYNC1,timestamp);
        content.put(Photo.SYNC1,0);
        content.put(Photo.SYNC2,photoURL.toString());
        content.put(Photo.MIMETYPE,Photo.CONTENT_ITEM_TYPE);
        addInsertOperation();
      //}
    }
    return this;
  }


  /***********************************************************************************************
  Adds the users politics
  @param politics the Politics
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addPolitics(String politics){
    content.clear();
    if (politics != null && !politics.equals("")) {
      content.put(PoliticsData.DATA,politics);
      content.put(PoliticsData.LABEL,context.getString(R.string.label_politics));
      content.put(Data.MIMETYPE,PoliticsData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a relationship
  @param type the type of relationship
  @param significantOther the significant other in the relationship
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addRelationship(String type,String significantOther){
    content.clear();
    if ((type != null && !type.equals("")) || (significantOther != null && !significantOther.equals(""))) {
      content.put(Data.MIMETYPE,RelationshipData.CONTENT_ITEM_TYPE);
      content.put(RelationshipData.LABEL,context.getString(R.string.label_relationship));
      content.put(RelationshipData.TYPE,type);
      content.put(RelationshipData.PERSON,significantOther);

      // Build the relationship message
      String relationshipMessage = "";

      if (type != null && !type.equals("")) {
        relationshipMessage += type;
        if (significantOther != null && !significantOther.equals("")) {
          relationshipMessage += " " + Language.get(R.string.to) + " " + significantOther;
        }
      }

      if (!relationshipMessage.equals("")) {
        content.put(RelationshipData.DATA,relationshipMessage);
      }

    }

    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds the users religion
  @param religion the Religion
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addReligion(String religion){
    content.clear();
    if (religion != null && !religion.equals("")) {
      content.put(ReligionData.DATA,religion);
      content.put(ReligionData.LABEL,context.getString(R.string.label_religion));
      content.put(Data.MIMETYPE,ReligionData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a "Facebook" status message
  @param message the message
  @param postingTime the time the post was made
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addStatus(String message,long postingTime){
    content.clear();
    if (message != null && !message.equals("")) {
      content.put(Data.MIMETYPE,StatusData.CONTENT_ITEM_TYPE);
      content.put(StatusData.LABEL,context.getString(R.string.label_status));
      content.put(StatusData.MESSAGE,message);
      content.put(StatusData.TIMESTAMP,postingTime);

      // Build the status message
      String statusMessage = "";

      if (message != null && !message.equals("")) {
        statusMessage += message;
      }

      if (postingTime > 0) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(postingTime);

        if (!statusMessage.equals("")) {
          statusMessage += " at " + formatTimestamp(calendar);
        }
      }

      if (!statusMessage.equals("")) {
        content.put(StatusData.DATA,statusMessage);
      }

    }

    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a "Facebook" status icon
  @param message the message
  @param postingTime the time the post was made
  @param dataId the ID of the status data
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addStatusIcon(String message,long postingTime,long dataId){
    content.clear();

    if (dataId != -1) {
      content.put(ContactsContract.StatusUpdates.DATA_ID,dataId);
      content.put(ContactsContract.StatusUpdates.STATUS,message);
      content.put(ContactsContract.StatusUpdates.STATUS_RES_PACKAGE,GlobalConstants.ACCOUNT_TYPE);
      content.put(ContactsContract.StatusUpdates.STATUS_LABEL,context.getString(R.string.status_label));
      content.put(ContactsContract.StatusUpdates.STATUS_ICON,R.drawable.small_icon);
      content.put(ContactsContract.StatusUpdates.STATUS_TIMESTAMP,postingTime);
    }

    if (content.size() > 0) {
      // insert() does a replace() for status messages
      operationBuilder = addInsertOperation(addCallerIsSyncAdapterParameter(ContactsContract.StatusUpdates.CONTENT_URI),yield);
      operationBuilder.withValues(content);

      yield = false;
      batchOperation.add(operationBuilder.build());
      //batchOperation.execute();
    }

    return this;

  }


  /***********************************************************************************************
  Adds an update operation into the batch
  ***********************************************************************************************/
  public void addUpdateOperation(Uri uri){
    operationBuilder = addUpdateOperation(uri,yield).withValues(content);
    yield = false;
    batchOperation.add(operationBuilder.build());
  }


  /***********************************************************************************************
  Adds the users backend server userId
  @param userID the userID on the backend server
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addUserId(long userId){
    content.clear();
    if (userId >= 0) {
      content.put(UserIdData.DATA,userId);
      content.put(UserIdData.LABEL,context.getString(R.string.label_userId));
      content.put(Data.MIMETYPE,UserIdData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds the username
  @param username the username
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addUsername(String username){
    content.clear();
    if (username != null && !username.equals("")) {
      content.put(Nickname.NAME,username);
      content.put(Nickname.TYPE,Nickname.TYPE_CUSTOM);
      content.put(Nickname.LABEL,context.getString(R.string.label_nickname));
      content.put(Data.MIMETYPE,Nickname.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds the website
  @param website the website URL
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addWebsite(String website){
    content.clear();
    if (website != null && !website.equals("")) {
      content.put(Website.URL,website);
      content.put(Website.TYPE,Website.TYPE_CUSTOM);
      content.put(Website.LABEL,context.getString(R.string.label_website));
      content.put(Data.MIMETYPE,Website.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Deleted the users photo
  @param contactUrl the Uri of the photo data entry
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations deletePhoto(Uri contactUrl){
    addDeleteOperation(contactUrl);
    return this;
  }



  /***********************************************************************************************
  Downloads a photo from a <code>URL</code>. If the image is too large, it is resized.
  @param photoURL the URL to the photo
  @return the <code>byte</code> array of photo data (in JPEG format)
  ***********************************************************************************************/
  public static byte[] downloadPhoto(URL photoURL){

    if (photoURL != null && !photoURL.toString().equals("")) {
      // Download the photo into a byte[] BLOB (stored as a String)
      byte[] photoBytes = new byte[1000];
      boolean photoSuccess = false;
      try {
        InputStream dataStream = photoURL.openStream();

        int readSize = 0;
        int receivedSize = dataStream.read(photoBytes,readSize,1000);

        while (receivedSize > 0) {
          // there is more data, so increase the array by another 1000 bytes and repeat the read
          readSize += receivedSize;
          photoBytes = ArrayResizer.resize(photoBytes,readSize + 1000);
          receivedSize = dataStream.read(photoBytes,readSize,1000);
        }

        dataStream.close();

        // resize to the actual size of the content
        photoBytes = ArrayResizer.resize(photoBytes,readSize);

        // Now check that the bytes are a valid JPEG file
        // (in case the URL was invalid or a 404 page was downloaded, etc)
        // (ie. only store the image if it is actually an image)
        Bitmap loadedJpg = BitmapFactory.decodeByteArray(photoBytes,0,readSize);
        if (loadedJpg != null) {
          // if the bitmap is larger than 640x640, resize it...
          int width = loadedJpg.getWidth();
          int height = loadedJpg.getHeight();
          // NOTE - Changed to 640x640 instead of 150x150 because there is a setting
          // where the user can choose to download small/medium/large photos. However,
          // we still set an upper limit of 640x640 so its not excessively big and taking
          // up heaps of storage space!
          if (width > 640 || height > 640) {
            width = 640;
            height = 640;
            // resize it
            loadedJpg = Bitmap.createScaledBitmap(loadedJpg,width,height,true);
            // get the new bytes for the resized image
            ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
            loadedJpg.compress(Bitmap.CompressFormat.JPEG,75,byteStream);
            photoBytes = byteStream.toByteArray();
          }
          photoSuccess = true;
        }
      }
      catch (Throwable t) {
        // Error downloading the photo
      }

      // Only add a photo if the download was successful
      if (photoSuccess) {
        return photoBytes;
      }

    }

    return null;
  }


  /***********************************************************************************************
  Formats a timestamp in a <code>Calendar</code> into a text <code>String</code>
  @param calendar the <code>Calendar</code> set to the timestamp
  @return the <code>String</code> representation of the timestamp
  ***********************************************************************************************/
  public String formatTimestamp(Calendar calendar){

    // Hour
    String timestamp = calendar.get(Calendar.HOUR) + ":";

    // Minute
    int minute = calendar.get(Calendar.MINUTE);
    if (minute < 10) {
      timestamp += "0";
    }
    timestamp += minute;

    // AM/PM
    if (calendar.get(Calendar.HOUR_OF_DAY) > 12) {
      timestamp += context.getString(R.string.time_pm) + " ";
    }
    else {
      timestamp += context.getString(R.string.time_am) + " ";
    }

    // Day
    int day = calendar.get(Calendar.DAY_OF_WEEK);
    if (day == Calendar.MONDAY) {
      timestamp += context.getString(R.string.day_monday) + " ";
    }
    else if (day == Calendar.TUESDAY) {
      timestamp += context.getString(R.string.day_tuesday) + " ";
    }
    else if (day == Calendar.WEDNESDAY) {
      timestamp += context.getString(R.string.day_wednesday) + " ";
    }
    else if (day == Calendar.THURSDAY) {
      timestamp += context.getString(R.string.day_thursday) + " ";
    }
    else if (day == Calendar.FRIDAY) {
      timestamp += context.getString(R.string.day_friday) + " ";
    }
    else if (day == Calendar.SATURDAY) {
      timestamp += context.getString(R.string.day_saturday) + " ";
    }
    else if (day == Calendar.SUNDAY) {
      timestamp += context.getString(R.string.day_sunday) + " ";
    }

    // Day number
    day = calendar.get(Calendar.DAY_OF_MONTH);
    timestamp += day;

    if (day == 1 || day == 21 || day == 31) {
      timestamp += context.getString(R.string.day_suffix_st) + " ";
    }
    else if (day == 2 || day == 22) {
      timestamp += context.getString(R.string.day_suffix_nd) + " ";
    }
    else if (day == 3 || day == 23) {
      timestamp += context.getString(R.string.day_suffix_rd) + " ";
    }
    else {
      timestamp += context.getString(R.string.day_suffix_th) + " ";
    }

    // Month
    int month = calendar.get(Calendar.MONTH);
    if (month == Calendar.JANUARY) {
      timestamp += context.getString(R.string.month_january) + " ";
    }
    else if (month == Calendar.FEBRUARY) {
      timestamp += context.getString(R.string.month_february) + " ";
    }
    else if (month == Calendar.MARCH) {
      timestamp += context.getString(R.string.month_march) + " ";
    }
    else if (month == Calendar.APRIL) {
      timestamp += context.getString(R.string.month_april) + " ";
    }
    else if (month == Calendar.MAY) {
      timestamp += context.getString(R.string.month_may) + " ";
    }
    else if (month == Calendar.JUNE) {
      timestamp += context.getString(R.string.month_june) + " ";
    }
    else if (month == Calendar.JULY) {
      timestamp += context.getString(R.string.month_july) + " ";
    }
    else if (month == Calendar.AUGUST) {
      timestamp += context.getString(R.string.month_august) + " ";
    }
    else if (month == Calendar.SEPTEMBER) {
      timestamp += context.getString(R.string.month_september) + " ";
    }
    else if (month == Calendar.OCTOBER) {
      timestamp += context.getString(R.string.month_october) + " ";
    }
    else if (month == Calendar.NOVEMBER) {
      timestamp += context.getString(R.string.month_november) + " ";
    }
    else if (month == Calendar.DECEMBER) {
      timestamp += context.getString(R.string.month_december) + " ";
    }

    // Year
    timestamp += calendar.get(Calendar.YEAR);

    return timestamp;
  }


  /***********************************************************************************************
  Updates the users biography
  @param biography the Biography
  @param oldBiography the old biography
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateBiography(String biography,String oldBiography,Uri contactUrl){
    if (biography != null && oldBiography != null && !biography.equals(oldBiography)) {
      content.clear();
      content.put(BiographyData.DATA,biography);
      content.put(Data.MIMETYPE,BiographyData.CONTENT_ITEM_TYPE);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the birthday
  @param birthday the birthday as MM/DD/YYYY or MM/DD/YYYY
  @param oldBirthday the old birthday as MM/DD/YYYY or MM/DD/YYYY
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateBirthday(String birthday,String oldBirthday,Uri contactUrl){
    content.clear();
    if (birthday != null && !birthday.equals("")){
      if (oldBirthday != null && birthday.equals(oldBirthday)) {
        // skip - identical
      }
      else {
        // changed
        content.put(Event.START_DATE,birthday);
        addUpdateOperation(contactUrl);
      }
    }
    else { // delete the entry because the birthday is blank
      addDeleteOperation(contactUrl);
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Converts a Birthday from Event.TYPE_CUSTOM to Event.TYPE_BIRTHDAY (v1.0 to v1.01)
  @param birthday the birthday as MM/DD/YYYY or MM/DD/YYYY
  @param oldBirthday the old birthday as MM/DD/YYYY or MM/DD/YYYY
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations convertBirthdayFromCustomToBirthday(String birthday,String oldBirthday,Uri contactUrl){
    content.clear();
    content.put(Event.START_DATE,birthday);
    content.put(Event.TYPE,Event.TYPE_BIRTHDAY);
    addUpdateOperation(contactUrl);
    return this;
  }


  /***********************************************************************************************
  Updates the checkin
  @param checkin the new checkin
  @param oldCheckin the old checkin
  @param checkinTime the checkin time
  @param oldCheckinTime the old checkin time
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateCheckin(String checkin,String oldCheckin,long checkinTime,long oldCheckinTime,Uri contactUrl){
    content.clear();

    // Build the checkin message
    String checkinMessage = "";

    if (checkin != null && !checkin.equals("")) {
      checkinMessage += checkin;
    }

    if (checkinTime > 0) {
      Calendar calendar = Calendar.getInstance();
      calendar.setTimeInMillis(checkinTime);

      if (!checkinMessage.equals("")) {
        checkinMessage += " at " + formatTimestamp(calendar);
      }
    }

    if (oldCheckin == null || (checkin != null && !checkin.equals(oldCheckin))) {
      content.put(CheckInData.VENUE,checkin);
    }

    if (checkinTime != oldCheckinTime || !checkinMessage.equals("")) {
      // Force the timestamp to be set if the checkinMessage has been set
      content.put(CheckInData.TIMESTAMP,checkinTime);

      Calendar calendar = Calendar.getInstance();
      calendar.setTimeInMillis(checkinTime);

      if (checkinMessage.equals("")) {
        // The timestamp has changed, so we need to force the updating of the check-in location
        content.put(CheckInData.VENUE,checkin);
      }
    }

    if (!checkinMessage.equals("")) {
      content.put(CheckInData.DATA,checkinMessage);
    }

    if (content.size() > 0) {
      content.put(Data.MIMETYPE,CheckInData.CONTENT_ITEM_TYPE);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates contact's email
  @param email the new email address
  @param oldEmail the old email address
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateEmail(String email,String oldEmail,Uri contactUrl){
    if (email != null && oldEmail != null && !email.equals(oldEmail)) {
      content.clear();
      content.put(Email.DATA,email);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the users gender
  @param gender the new gender
  @param oldGender the old gender
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateGender(String gender,String oldGender,Uri contactUrl){
    if (gender != null && oldGender != null && !gender.equals(oldGender)) {
      content.clear();
      content.put(GenderData.DATA,gender);
      content.put(Data.MIMETYPE,GenderData.CONTENT_ITEM_TYPE);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the home city
  @param homeCity the new home city
  @param oldHomeCity the old home city
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateHomeCity(String homeCity,String oldHomeCity,Uri contactUrl){
    content.clear();
    if (homeCity != null && oldHomeCity != null && !homeCity.equals(oldHomeCity)) {
      content.put(StructuredPostal.FORMATTED_ADDRESS,homeCity);

      int commaPosition = homeCity.indexOf(',');
      if (commaPosition > 0) {
        String city = homeCity.substring(0,commaPosition);
        content.put(StructuredPostal.CITY,city);
        String state = homeCity.substring(commaPosition + 1).trim();
        content.put(StructuredPostal.REGION,state);
      }

    }

    if (content.size() > 0) {
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the location
  @param location the new location
  @param oldLocation the old location
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateLocation(String location,String oldLocation,Uri contactUrl){
    content.clear();
    if (location != null && oldLocation != null && !location.equals(oldLocation)) {
      content.put(StructuredPostal.FORMATTED_ADDRESS,location);

      int commaPosition = location.indexOf(',');
      if (commaPosition > 0) {
        String city = location.substring(0,commaPosition);
        content.put(StructuredPostal.CITY,city);
        String state = location.substring(commaPosition + 1).trim();
        content.put(StructuredPostal.REGION,state);
      }

    }

    if (content.size() > 0) {
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates contact's name
  @param firstName the new first name
  @param oldFirstName the old first name
  @param middleName the new middle name
  @param oldMiddleName the old middle name
  @param lastName the new last name
  @param oldLastName the old last name
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateName(String firstName,String oldFirstName,String middleName,String oldMiddleName,String lastName,String oldLastName,Uri contactUrl){
    content.clear();
    if (firstName != null && oldFirstName != null && !firstName.equals(oldFirstName)) {
      content.put(StructuredName.GIVEN_NAME,firstName);
    }
    if (middleName != null && oldMiddleName != null && !middleName.equals(oldMiddleName)) {
      content.put(StructuredName.MIDDLE_NAME,middleName);
    }
    if (lastName != null && oldLastName != null && !lastName.equals(oldLastName)) {
      content.put(StructuredName.FAMILY_NAME,lastName);
    }
    if (content.size() > 0) {
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the users photo
  @param photoURL the new photo URL
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updatePhoto(URL photoURL,Uri contactUrl){
    if (photoURL != null && !photoURL.toString().equals("")) {
      content.clear();

      // Download the photo into a byte[] BLOB (stored as a String)
      //byte[] photoBytes = downloadPhoto(photoURL);

      // Only update the photo if the download was successful
      //if (photoBytes != null && photoBytes.length > 0) {
        // Store the photo
        //content.put(Photo.PHOTO,photoBytes);
        
        // v1.3
        // Added the timestamp here so that it updates the timestamp when the photo is downloaded.
        // Otherwise it would just update every sync time because the photo would be 'old'.
        //String timestamp = "" + Calendar.getInstance().getTimeInMillis();
        //content.put(Photo.SYNC1,timestamp);
        content.put(Photo.SYNC2,photoURL.toString());
        addUpdateOperation(contactUrl);
      //}

    }
    return this;
  }


  /***********************************************************************************************
  Updates the users politics
  @param politics the politics
  @param oldPolitics the old politics
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updatePolitics(String politics,String oldPolitics,Uri contactUrl){
    if (politics != null && oldPolitics != null && !politics.equals(oldPolitics)) {
      content.clear();
      content.put(PoliticsData.DATA,politics);
      content.put(Data.MIMETYPE,PoliticsData.CONTENT_ITEM_TYPE);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Update a relationship
  @param type the type of relationship
  @param oldType the old type of relationship
  @param significantOther the significant other in the relationship
  @param oldSignificantOther the old significant other in the relationship
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateRelationship(String type,String oldType,String significantOther,String oldSignificantOther,Uri contactUrl){
    content.clear();
    if ((type != null && !type.equals(oldType)) || (significantOther != null && !significantOther.equals(oldSignificantOther))) {
      content.put(Data.MIMETYPE,RelationshipData.CONTENT_ITEM_TYPE);
      content.put(RelationshipData.TYPE,type);
      content.put(RelationshipData.PERSON,significantOther);

      // Build the relationship message
      String relationshipMessage = "";

      if (type != null && !type.equals("")) {
        relationshipMessage += type;
        if (significantOther != null && !significantOther.equals("")) {
          relationshipMessage += " " + Language.get(R.string.to) + " " + significantOther;
        }
      }

      if (!relationshipMessage.equals("")) {
        content.put(RelationshipData.DATA,relationshipMessage);
      }

    }

    if (content.size() > 0) {
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the users religion
  @param religion the religion
  @param oldReligion the old religion
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateReligion(String religion,String oldReligion,Uri contactUrl){
    if (religion != null && oldReligion != null && !religion.equals(oldReligion)) {
      content.clear();
      content.put(ReligionData.DATA,religion);
      content.put(Data.MIMETYPE,ReligionData.CONTENT_ITEM_TYPE);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates a "Facebook" status message
  @param message the users new status message
  @param oldMessage the old message
  @param postingTime the posting time of the message
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateStatus(String message,String oldMessage,long postingTime,Uri contactUrl){

    if (message == null || oldMessage == null || (message != null && oldMessage != null && !message.equals(oldMessage))) {

      // Build the status message
      String statusMessage = "";

      if (message != null && !message.equals("")) {
        statusMessage += message;
      }

      if (postingTime > 0) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(postingTime);

        if (!statusMessage.equals("")) {
          statusMessage += " at " + formatTimestamp(calendar);
        }
      }

      content.put(StatusData.DATA,statusMessage);
      content.put(StatusData.MESSAGE,message);
      content.put(StatusData.TIMESTAMP,postingTime);
      content.put(Data.MIMETYPE,StatusData.CONTENT_ITEM_TYPE);
      addUpdateOperation(contactUrl);
    }

    return this;

  }


  /***********************************************************************************************
  Updates the "Facebook" status icon
  @param message the new message
  @param message the old message
  @param postingTime the time the post was made
  @param dataId the ID of the status data
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateStatusIcon(String message,String oldMessage,long postingTime,long dataId){
    if (message == null || oldMessage == null || (message != null && oldMessage != null && !message.equals(oldMessage))) {
      return addStatusIcon(message,postingTime,dataId);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the users backend server user ID
  @param userId the new user ID
  @param oldUserId the old user ID
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateUserId(long userId,long oldUserId,Uri contactUrl){
    if (userId >= 0 && oldUserId >= 0 && userId != oldUserId) {
      content.clear();
      content.put(UserIdData.DATA,userId);
      content.put(Data.MIMETYPE,UserIdData.CONTENT_ITEM_TYPE);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the username
  @param username the username
  @param oldUsername the old username
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateUsername(String username,String oldUsername,Uri contactUrl){
    if (username != null && oldUsername != null && !username.equals(oldUsername)) {
      content.clear();
      content.put(Nickname.NAME,username);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the users website
  @param website the website
  @param oldWebsite the old website
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateWebsite(String website,String oldWebsite,Uri contactUrl){
    if (website != null && oldWebsite != null && !website.equals(oldWebsite)) {
      content.clear();
      content.put(Website.URL,website);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Adds a CALLER_IS_SYNCADAPTER parameter
  ***********************************************************************************************/
  public static Uri addCallerIsSyncAdapterParameter(Uri uri){
    return uri.buildUpon().appendQueryParameter(ContactsContract.CALLER_IS_SYNCADAPTER,"true").build();
  }


  /***********************************************************************************************
  Builds a <code>ContentProviderOperation</code> for a delete operation
  ***********************************************************************************************/
  public static ContentProviderOperation.Builder addDeleteOperation(Uri uri,boolean yield){
    return ContentProviderOperation.newDelete(addCallerIsSyncAdapterParameter(uri)).withYieldAllowed(yield);

  }


  /***********************************************************************************************
  Builds a <code>ContentProviderOperation</code> for an insert operation
   ***********************************************************************************************/
  public static ContentProviderOperation.Builder addInsertOperation(Uri uri,boolean yield){
    return ContentProviderOperation.newInsert(addCallerIsSyncAdapterParameter(uri)).withYieldAllowed(yield);
  }


  /***********************************************************************************************
  Builds a <code>ContentProviderOperation</code> for an update operation
   ***********************************************************************************************/
  public static ContentProviderOperation.Builder addUpdateOperation(Uri uri,boolean yield){
    return ContentProviderOperation.newUpdate(addCallerIsSyncAdapterParameter(uri)).withYieldAllowed(yield);
  }


  /***********************************************************************************************
  Creates a new <code>ContactOperations</code> for adding new contact to the Android Contacts
  @param context the Authenticator Activity context
  @param userId the userId of the sample SyncAdapter user object
  @param accountName the username of the current login
  @param batchOperations any batch operations to perform when creating the new contact 
  @return instance of ContactOperations
  ***********************************************************************************************/
  public static ContactOperations createNewContact(Context context,int userId,String accountName,BatchOperation batchOperations){
    return new ContactOperations(context,userId,accountName,batchOperations);
  }


  /***********************************************************************************************
  Creates a new <code>ContactOperations</code> for adding new group to the Android Contacts
  @param context the Authenticator Activity context
  @param groupId the groupId of the sample SyncAdapter user object
  @param accountName the username of the current login
  @param batchOperations any batch operations to perform when creating the new contact 
  @return instance of ContactOperations
  ***********************************************************************************************/
  public static ContactOperations createNewGroup(Context context,int groupId,String accountName,BatchOperation batchOperations){
    return new ContactOperations(context,groupId,accountName,batchOperations,true);
  }


  /***********************************************************************************************
  Creates a new <code>ContactOperations</code> for updating an existing Android Contact
  @param context the Authenticator Activity context
  @param rawContactId the unique ID of the existing contact
  @param batchOperations any batch operations to perform when updating the contact 
  @return instance of ContactOperations
   ***********************************************************************************************/
  public static ContactOperations updateExistingContact(Context context,long rawContactId,BatchOperation batchOperations){
    return new ContactOperations(context,rawContactId,batchOperations);
  }

}
