////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.facebook.security;

import org.watto.android.Settings;


/***********************************************************************************************
Convenience methods for establishing OAuth 2.0 authentication to a server
***********************************************************************************************/
public abstract class OAuth2Client {

  /** The OAuth key for this application **/
  String consumerKey = "";
  /** The OAuth secret for this application **/
  String consumerSecret = "";
  /** The callback URL for this application **/
  String callbackUrl = "watto://callback";
  /** The authentication code **/
  String code = "";
  /** The access token **/
  String accessToken = "";


  /***********************************************************************************************
  Initialises the OAuth provider with the <i>consumerKey</i>, <i>consumerSecret</i> and the 
  <i>callbackUrl</i> for the application
  @param consumerKey the OAuth key for this application
  @param consumerSecret the OAuth secret for this application
  @param callbackUrl the OAuth callback URL for this application
  ***********************************************************************************************/
  public OAuth2Client(String consumerKey, String consumerSecret, String callbackUrl){
    this.consumerKey = consumerKey;
    this.consumerSecret = consumerSecret;
    this.callbackUrl = callbackUrl;
  }


  /***********************************************************************************************
  Gets the access token
  @return the <i>accessToken</i>
  ***********************************************************************************************/
  public String getAccessToken(){
    return accessToken;
  }


  /***********************************************************************************************
  Gets the URL for converting an authentication <i>code</i> into an access token
  @param code the authentication code
  @return the URL used to generate the access token
  ***********************************************************************************************/
  public abstract String getAccessTokenUrl(String code);


  /***********************************************************************************************
  Gets the URL for authenticating the application
  @return the URL used to authenticate the application
  ***********************************************************************************************/
  public abstract String getAuthenticationUrl();


  /***********************************************************************************************
  Gets the callback URL
  @return the <i>callbackUrl</i>
  ***********************************************************************************************/
  public String getCallbackUrl(){
    return callbackUrl;
  }


  /***********************************************************************************************
  Gets the authentication code
  @return the <i>code</i>
  ***********************************************************************************************/
  public String getCode(){
    return code;
  }


  /***********************************************************************************************
  Gets the OAuth key for this application
  @return the <i>consumerKey</i>
  ***********************************************************************************************/
  public String getConsumerKey(){
    return consumerKey;
  }


  /***********************************************************************************************
  Gets the OAuth secret for this application
  @return the <i>consumerSecret</i>
  ***********************************************************************************************/
  public String getConsumerSecret(){
    return consumerSecret;
  }


  /***********************************************************************************************
  Loads the authentication <i>code</i> and the <i>accessToken</i> from the <code>Settings</code>
  ***********************************************************************************************/
  public void loadFromSettings(){
    // Load the code and access token from the Settings
    if (Settings.hasApplication()) {
      this.code = Settings.getString("OAuth2Code");
      this.accessToken = Settings.getString("OAuth2AccessToken");
    }
  }


  /***********************************************************************************************
  Converts the authentication <i>code</i> into an access token
  @param code the authentication code
  @return the access token
  ***********************************************************************************************/
  public abstract String retreiveAccessToken(String code);


  /***********************************************************************************************
  Sets the access token, and stores it in the <code>Settings</code>
  @param accessToken the access token
  ***********************************************************************************************/
  public void setAccessToken(String accessToken){
    this.accessToken = accessToken;

    // Store the access token in the Settings
    if (Settings.hasApplication()) {
      Settings.set("OAuth2AccessToken",accessToken);
    }
  }


  /***********************************************************************************************
  Sets the callback URL
  @param callbackUrl the callback URL
  ***********************************************************************************************/
  public void setCallbackUrl(String callbackUrl){
    this.callbackUrl = callbackUrl;
  }


  /***********************************************************************************************
  Sets the authentication code, and stores it in the <code>Settings</code>
  @param code the authentication code
  ***********************************************************************************************/
  public void setCode(String code){
    this.code = code;

    // Store the code in the Settings
    if (Settings.hasApplication()) {
      Settings.set("OAuth2Code",code);
    }
  }


  /***********************************************************************************************
  Sets the OAuth key for this application
  @param consumerKey the OAuth key
  ***********************************************************************************************/
  public void setConsumerKey(String consumerKey){
    this.consumerKey = consumerKey;
  }


  /***********************************************************************************************
  Sets the OAuth secret for this application
  @param consumerSecret the OAuth secret
  ***********************************************************************************************/
  public void setConsumerSecret(String consumerSecret){
    this.consumerSecret = consumerSecret;
  }
}
