////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.facebook.sync;

import java.net.URL;
import java.util.ArrayList;
import java.util.Calendar;
import org.watto.android.Language;
import org.watto.android.Settings;
import org.watto.program.android.sync.facebook.ContactOperations;
import org.watto.program.android.sync.facebook.GlobalConstants;
import org.watto.program.android.sync.facebook.R;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.ContentProviderOperation;
import android.content.ContentResolver;
import android.content.Intent;
import android.database.Cursor;
import android.os.AsyncTask;
import android.os.Binder;
import android.os.IBinder;
import android.provider.ContactsContract;
import android.provider.ContactsContract.CommonDataKinds.Photo;
import android.provider.ContactsContract.Data;
import android.provider.ContactsContract.RawContacts;


/***********************************************************************************************
A <code>Service</code> for downloading the photos for each contact, separately to the actual
sync operation.
***********************************************************************************************/
public class PhotoDownloadService extends Service {

  NotificationManager notificationManager;

  /** The ID for all PhotoDownload-related notifications **/
  public static int DOWNLOAD_NOTIFICATION = 0;

  /** The Binder object that receives interactions from other clients **/
  private final IBinder binder = new PhotoDownloadServiceBinder();


  /***********************************************************************************************
  Constructor
  ***********************************************************************************************/
  public PhotoDownloadService(){
    super();
  }


  /***********************************************************************************************
  Gets the <code>IBinder</code> to this <code>Service</code>
  @param intent the <code>Intent</code> that is being bound
  @return the <i>binder</i>
  ***********************************************************************************************/
  public IBinder onBind(Intent intent){
    return binder;
  }


  /***********************************************************************************************
  Creates the <code>Service</code>
  ***********************************************************************************************/
  public void onCreate(){
    notificationManager = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
  }


  /***********************************************************************************************
  Finishes the <code>Service</code>
  ***********************************************************************************************/
  public void onDestroy(){
    //notificationManager.cancel(FLICKR_NOTIFICATION);
  }


  /***********************************************************************************************
  Runs the <code>Service</code>
  ***********************************************************************************************/
  public int onStartCommand(Intent intent,int flags,int startId){

    // Run all the processing in a separate Thread so that the Service returns without timing out
    runDownloadPhotoTask();

    // The service will be killed when it has finished - no point leaving it hang around
    return START_NOT_STICKY;
  }


  /***********************************************************************************************
  Shows a <code>Notification</code> whilever this <code>Service</code> is running
  @param notificationType the type of notification to display
  ***********************************************************************************************/
  public Notification showNotification(){

    // Prepare the notification properties
    int icon = android.R.drawable.stat_sys_download;
    String tickerText = Language.get(R.string.notification_download_photos_ticker_text);
    String title = Language.get(R.string.notification_download_photos_title);
    String longText = Language.get(R.string.notification_download_photos_long_text);
    long when = System.currentTimeMillis();
    
    // v1.31 - Fix for Icecream Sandwich
    //Intent notificationIntent = null;
    Intent notificationIntent = new Intent();

    // Cancel any current download notifications
    notificationManager.cancel(DOWNLOAD_NOTIFICATION);

    // Show the new notification
    Notification notification = new Notification(icon,tickerText,when);


    PendingIntent contentIntent = null;
    try {
      contentIntent = PendingIntent.getActivity(this,0,notificationIntent,PendingIntent.FLAG_CANCEL_CURRENT);
    }
    catch (IllegalArgumentException e) {
    }

    notification.setLatestEventInfo(getApplicationContext(),title,longText,contentIntent);
    notificationManager.notify(DOWNLOAD_NOTIFICATION,notification);

    return notification;
  }

  /***********************************************************************************************
  Binder class for remote access
  ***********************************************************************************************/
  public class PhotoDownloadServiceBinder extends Binder {

    PhotoDownloadService getService(){
      return PhotoDownloadService.this;
    }
  }

  /***********************************************************************************************
  A background Thread for downloading the photo (so that the Service doesn't time out)
   ***********************************************************************************************/
  protected class DownloadPhotoTask extends AsyncTask<Object,Object,Object> {


    /***********************************************************************************************
    Downloads the photos in the background
    @param arguments the arguments passed to the background task
    @return the result of the background operation
     ***********************************************************************************************/
    @SuppressWarnings("unused")
    protected Object doInBackground(Object ... arguments){

      // TODO REMOVE THIS LINE FOR TESTING USING THE DEBUGGER!!!
      //android.os.Debug.waitForDebugger();
      
      // Alert that the download has started
      boolean notify = false;
      if (!Settings.has("ShowPhotoDownloadNotification")){
        notify = true;
      }
      else {
        notify = Settings.getBoolean("ShowPhotoDownloadNotification");
      }
      
      if (notify){
        showNotification();
      }

      // Find the photo objects for each friend, then trigger the download
      String[] columns = new String[]{Data._ID,Photo.SYNC1,Photo.SYNC2};
      String whereClause = RawContacts.ACCOUNT_TYPE + "='" + GlobalConstants.ACCOUNT_TYPE + "' AND " + Data.MIMETYPE + "='" + Photo.CONTENT_ITEM_TYPE + "'";

      ContentResolver resolver = getApplicationContext().getContentResolver();
      
      // The batch list of updates
      ArrayList<ContentProviderOperation> batchUpdate = new ArrayList<ContentProviderOperation>();
      
      Cursor c = resolver.query(Data.CONTENT_URI,columns,whereClause,null,null);
      try {
        // for each contact...
        while (c.moveToNext()) {
          // The data ID for the photo
          long columnId = c.getLong(0);
          
          // The timestamp of the last download
          long timestamp = Calendar.getInstance().getTimeInMillis();
          boolean timeExpired = false;
          try {
            long oldTimestamp = Long.parseLong(c.getString(1));
            if (oldTimestamp <= 300000){ // new contacts should get their photo updated immediately. 5 mins in milliseconds
              timeExpired = true;
            }
            else {
              // Check that the photo is at least 1 day old
              long timeDifference = timestamp - oldTimestamp;
              if (timeDifference > 82800000) { // 23 hours in milliseconds
                timeExpired = true;
              }
            }
          }
          catch (NumberFormatException e) {
            timeExpired = true;
          }
          
          if (!timeExpired){
            // < 24 hours since the last update, so skip this photo until next time
            continue;
          }
            
          
          // The photo URL
          String photoURL = c.getString(2);
          if (photoURL == null || photoURL.equals("")){
            continue;
          }
          
          // Download the photo into a byte[] BLOB (stored as a String)
          byte[] photoBytes = ContactOperations.downloadPhoto(new URL(photoURL));
          if (photoBytes != null && photoBytes.length > 0) {
            
            // add a batch update
            batchUpdate.add(ContentProviderOperation.newUpdate(Data.CONTENT_URI)
                .withSelection(Data._ID + "=" + columnId,null)
                .withValue(Photo.SYNC1,timestamp)
                .withValue(Photo.PHOTO,photoBytes)
                .build());
          }
          
        }
      }
      catch (Throwable t){
        String error = "";
      }
      finally {
        if (c != null) {
          c.close();
        }
      }
      
      // perform the actual updates
      try {
        resolver.applyBatch(ContactsContract.AUTHORITY, batchUpdate);
      }
      catch (Throwable t){
        String error = "";
      }

      
      // Remove the notification when the download has finished
      if (notify){
        notificationManager.cancel(DOWNLOAD_NOTIFICATION);
      }

      stopSelf();

      return true;
    }


    /***********************************************************************************************
    Does nothing
    @param result the result of the background operation
     ***********************************************************************************************/
    protected void onPostExecute(Object result){
      // Nothing
    }

  }


  /***********************************************************************************************
  Runs a <code>AsyncTask</code> that downloads the photos
  ***********************************************************************************************/
  public void runDownloadPhotoTask(){
    new DownloadPhotoTask().execute();
  }

}
